
/** INCLUDES *******************************************************/
#include "system.h"

#include <stdint.h>
#include <stdio.h>
#include <string.h>
#include <stddef.h>

#include "usb.h"

#include "app_led_usb_status.h"
#include "app_device_cdc_basic.h"
#include "usb_config.h"

/** VARIABLES ******************************************************/

static bool buttonPressed;
static char buttonMessage[] = "Button pressed.\r\n";
static uint8_t readBuffer[CDC_DATA_OUT_EP_SIZE];
static uint8_t writeBuffer[CDC_DATA_IN_EP_SIZE];

/*********************************************************************
* Overview: Initializes the demo code
********************************************************************/
void APP_DeviceCDCBasicDemoInitialize()
{   
    line_coding.bCharFormat = 0;
    line_coding.bDataBits = 8;
    line_coding.bParityType = 0;
    line_coding.dwDTERate = 9600;

    buttonPressed = false;
}

/*********************************************************************
* Overview: Keeps the demo running.
********************************************************************/
void APP_DeviceCDCBasicDemoTasks()
{   
    if( USBGetDeviceState() < CONFIGURED_STATE ) return;
    if( USBIsDeviceSuspended()== true ) return;
 
    if(mUSBUSARTIsTxTrfReady() == true){
        if(cgToMark) {
            putrsUSBUSART("buttonMessage");
            cgToMark = false;
        }
    }
    

    /* *********************************** */
    if( USBUSARTIsTxTrfReady() == true){
        uint8_t i;
        uint8_t numBytesRead;

        numBytesRead = getsUSBUSART(readBuffer, sizeof(readBuffer));

        /* For every byte that was read... */
        for(i=0; i<numBytesRead; i++){
            switch(readBuffer[i]){
                case 0x0A:
                case 0x0D:
                    writeBuffer[i] = readBuffer[i];
                    break;
                default:
                    writeBuffer[i] = readBuffer[i] + 1;
                    break;
            }
        }

        if(numBytesRead > 0){
            putUSBUSART(writeBuffer,numBytesRead);
        }
    }

    CDCTxService();
}

/* ------------------------------------------------------
 * UserCW_Init();
 * CWトレーナー関連の初期化
 ------------------------------------------------------ */
void UserCW_Init(){
    
    LATC = 0;
    TRISC6 = 0;TRISC1 = 0;  // LED6, LED1
    
    TRISB7 = 0;             // Speaker
    TRISB5 = 1;             // Key input
    ANS11  = 0;             // RB5 digital
    WPUB5  = 1;             // RB5 弱pullup
    RABPU  = 0;             //     弱pullup
    // Timer 2 設定  48MHz/4 /4/250/12 = 1KHz(1ms)
    T2CON  = 0b01011101;    // Post 1/12　Pre_1/4
    PR2 = 249;              // T2 PR設定
    
    IPR1bits.TMR2IP = 0;    // Timer2 からの割込みを低優先に設定
    PIR1bits.TMR2IF = 0;    // Timer2 からの割込みフラッグをクリア
    PIE1bits.TMR2IE = 1;    // Timer2 からの割込みを許可

    RCONbits.IPEN=1;        // ２段階の割込みに設定
    INTCONbits.GIEH=1;      // 高優先割込みを許可
    INTCONbits.GIEL=1;      // 低優先割込みを許可

}

/* ------------------------------------------------------
 * UserCW_Tasks();
 * CWトレーナー関連の本体
 ------------------------------------------------------ */
void UserCW_Tasks(){
    if( USBGetDeviceState() < CONFIGURED_STATE ) return;
    if( USBIsDeviceSuspended()== true ) return;

    // key 押された時の処理
    if( USBUSARTIsTxTrfReady() == true){
        if(cgToMark){                   // Key　押されたなら
            cgToMark = 0;                   // 押されたイベントをOFF
            if(onGoing){                    // 最初は時間の出力なし
                if(modeDsp){
                    sprintf(writeBuffer," %5d\r\n",SP_time);
                    putUSBUSART(writeBuffer,8);
                }else{
                    sprintf(writeBuffer,"S %5d\r\n",SP_time);
                    putUSBUSART(writeBuffer,9);
                }
            }else{                          // 初回はタイトル表示
                if(modeDsp){                // 組み合わせモードのみ
                    sprintf(writeBuffer,"    Mark  Space (mS)\r\n");
                    putUSBUSART(writeBuffer,22);
                }
            }
            onGoing = 1;                    // 継続中フラッグセット
        }
        else if(cgToSpace){             // Key　離れたなら
            cgToSpace = 0;                  // 離れたイベントをOFF
            if(modeDsp){
                sprintf(writeBuffer,"%2d %5d ",numMark++,MK_time);
                putUSBUSART(writeBuffer,9);
            }else{
                sprintf(writeBuffer,"M %5d\r\n",MK_time);
                putUSBUSART(writeBuffer,9);
            }
        }
        else if(gapDetected){           // Key操作　中断なら
            gapDetected = 0;                // GAPイベントをOFF
            onGoing = 0;                    // 継続フラグリセット
            numMark = 1;                    // 符号番号リセット
            if(modeDsp){
                sprintf(writeBuffer,"   ---\r\n\r\n");
                putUSBUSART(writeBuffer,10);
            }else{
                sprintf(writeBuffer,"G   ---\r\n");
                putUSBUSART(writeBuffer,9);
            }
        }
    }

    // CDCドライバーの送信の準備ができていることを確認
    if( USBUSARTIsTxTrfReady() == true)
    {
        uint8_t i;
        uint8_t numBytesRead;

        numBytesRead = getsUSBUSART(readBuffer, sizeof(readBuffer));
        
//        j=0;
        writeBuffer[0] = '\0';
//        memset(writeBuffer, '\0', sizeof(writeBuffer));
        // すべての読み取りバイトに対し以下を繰り返す
        for(i=0; i<numBytesRead; i++){
            switch(readBuffer[i]) {
                // Vコマンドを受け取った場合は、Versionコードを返送
                case 'v':
                case 'V':
                    sprintf(writeBuffer,"%sV iwa2104\r\n",writeBuffer);
//                    j += 11;
                    break;
                // 4,5 コマンドを受け取った場合は、表示モードを変更
                case '4':
                    modeDsp = 0;
                    sprintf(writeBuffer,"%s4 Separated format\r\n",writeBuffer);
//                    j += 20;
                    break;
                case '5':
                    modeDsp = 1;
                    sprintf(writeBuffer,"%s5 Combined format\r\n",writeBuffer);
//                    j += 19;
                    break;
            }
        }
        // 受信したすべてのデータを処理した後、返送データを送信する
        if(numBytesRead > 0)
        {
            putUSBUSART(writeBuffer,strlen(writeBuffer));
        }
    }

    // デバイスからホストへのトランザクションを処理します。
    // CDCTxService()を定期的に呼び出さないと、
    // データがUSBホストに送信されません
    CDCTxService();

}

//-----------------低優先割込みの処理------------------------
void __interrupt(low_priority) Lo_Isr(void){    // xc8 ver2.x用
    
    static uint16_t keyTimer  = 0; // チャタリング監視回数
    static uint8_t  keyMonCnt = 0; // チャタリング監視回数
    static uint8_t  keyClean  = 1;    // クリーンkey信号
    
    if(PIR1bits.TMR2IF){            //Timer2からの割込みを確認
        PIR1bits.TMR2IF = 0;        // 割込みフラッグをクリア
        // =========================
        // Key時間計測タイマー処理 ---------------------
        if(onGoing)keyTimer++;      // 電鍵操作中ならタイマーを進める
        if(keyTimer > 2000){        // 2秒以上なら
            gapDetected = 1;        //   ギャップ操作終了なので
            keyTimer = 0;           // 電鍵操作中ならタイマーを進める
        }
        // チャタリング防止 Key-up で keyClean=1 ---------------------
        if(keyMonCnt == 0){             // keyが安定状態時
            if(keyClean != keyIn)           // keyの状態変更で
                keyMonCnt = MonCnt;         // 監視カウントをセット
        }else{                          // チャタリング監視中に
            if(keyClean != keyIn){          // key状態が変化のままなら
                keyMonCnt--;                //   変化継続を20回確認
                if(keyMonCnt == 0){         // 監視期間終了で
                    keyClean = keyIn;       // 新しい状態を反映
                    if(keyClean){           // upなら
                        cgToSpace = 1;      //  「離した」を[1]
                        MK_time = keyTimer; //   Mark時間を記録
                    }else{                  // downなら
                        cgToMark = 1;       //  「押した」を[1]
                        SP_time = keyTimer; //   Space時間を記録
                    }
                    keyTimer = 0;           // Timerリセット
                }       
            }else{                          // keyが元に戻ったら
                keyMonCnt = 0;              //   keyが安定状態を維持
            }
        }
        // サイドトーン -------------------------
        sp_LAT ^= 1;                // 1ms毎に反転 = 500Hz
        if(keyIn)toneOFF();         // key を叩くと
            else tone_ON();         //     サイドトーンが出る
        // サイドトーン END ---------------------
    }
}
